import {expect} from 'chai'
import sinon from 'sinon'
import segmentWrapper from '../src/analytics/segmentWrapper'

const dmpTrack = require('../src/analytics/dmp/track')

describe('#segmentWrapper.track', () => {
  let dmpTrackStub
  let dmpOnLoadStub
  const trackArgs = [
    'fakeEvent',
    {fakePropKey: 'fakePropValue'},
    {
      fakeOptionKey: 'fakeOptionValue',
      traits: {fakeTraitKey: 'fakeTraitValue'},
      integrations: {fakeIntegrationKey: 'fakeIntegrationValue'}
    }
  ]

  const visitorId = 'fakeVisitorId'

  beforeEach(() => {
    dmpTrackStub = sinon.stub(dmpTrack, 'default').resolves(true)
    dmpOnLoadStub = sinon.stub(dmpTrack, 'onLoad').resolves(true)
    window.Visitor = {}
    window.Visitor.getInstance = sinon.stub().returns({
      getMarketingCloudVisitorID: sinon.stub().returns('fakeCloudId')
    })

    window.analytics = {
      ready: function(cb) {
        return new Promise((resolve, reject) => {
          resolve(cb())
        })
      },
      user: () => ({
        anonymousId: () => 'fakeAnonymousId',
        id: () => 'fakeId'
      }),
      track: sinon.stub()
    }
  })

  afterEach(() => {
    delete window.Visitor
    delete window.analytics
    dmpTrackStub.restore()
    dmpOnLoadStub.restore()
  })

  describe('when the dmp is enabled', () => {
    let isDMPReadyStub
    beforeEach(() => {
      isDMPReadyStub = () => Promise.resolve(true)
    })

    it('should set the visitor id as adobe analytics trait', done => {
      segmentWrapper({visitorId, isDMPReady: isDMPReadyStub})
        .track(...trackArgs)
        .then(() => {
          expect(
            window.analytics.track.firstCall.args[2].integrations[
              'Adobe Analytics'
            ].marketingCloudVisitorId
          ).to.equal('fakeVisitorId')
          done()
        })
        .catch(done)
    })

    it('should set the right site prop', done => {
      segmentWrapper({visitorId, isDMPReady: isDMPReadyStub})
        .track(...trackArgs)
        .then(() => {
          expect(window.analytics.track.firstCall.args[1].site).to.equal(
            'infojobs.net'
          )
          done()
        })
        .catch(done)
    })

    it('should add anonymousId as options trait', done => {
      segmentWrapper({visitorId, isDMPReady: isDMPReadyStub})
        .track(...trackArgs)
        .then(() => {
          expect(
            window.analytics.track.firstCall.args[2].traits.anonymousId
          ).to.equal('fakeAnonymousId')
          done()
        })
        .catch(done)
    })

    it('should add the ssf trait as enabled', done => {
      segmentWrapper({visitorId, isDMPReady: isDMPReadyStub})
        .track(...trackArgs)
        .then(() => {
          expect(window.analytics.track.firstCall.args[2].traits.ssf).to.equal(
            0
          )
          done()
        })
        .catch(done)
    })
  })

  describe('when the dmp is disabled', () => {
    let isDMPReadyStub
    beforeEach(() => {
      isDMPReadyStub = () => Promise.resolve(false)
    })

    it('should add the ssf trait as disabled', done => {
      segmentWrapper({visitorId, isDMPReady: isDMPReadyStub})
        .track(...trackArgs)
        .then(() => {
          expect(window.analytics.track.firstCall.args[2].traits.ssf).to.equal(
            1
          )
          done()
        })
        .catch(done)
    })
  })
})
